@@ -46,6 +46,7 @@ ALL_ROUTES = [
     ('virtual:credentials', 'SoftLayer.CLI.virt.credentials:cli'),
     ('virtual:capacity', 'SoftLayer.CLI.virt.capacity:cli'),
     ('virtual:placementgroup', 'SoftLayer.CLI.virt.placementgroup:cli'),
+    ('virtual:migrate', 'SoftLayer.CLI.virt.migrate:cli'),
 
     ('dedicatedhost', 'SoftLayer.CLI.dedicatedhost'),
     ('dedicatedhost:list', 'SoftLayer.CLI.dedicatedhost.list:cli'),
@@ -236,6 +237,7 @@ ALL_ROUTES = [
     ('hardware:detail', 'SoftLayer.CLI.hardware.detail:cli'),
     ('hardware:billing', 'SoftLayer.CLI.hardware.billing:cli'),
     ('hardware:edit', 'SoftLayer.CLI.hardware.edit:cli'),
+    ('hardware:guests', 'SoftLayer.CLI.hardware.guests:cli'),
     ('hardware:list', 'SoftLayer.CLI.hardware.list:cli'),
     ('hardware:power-cycle', 'SoftLayer.CLI.hardware.power:power_cycle'),
     ('hardware:power-off', 'SoftLayer.CLI.hardware.power:power_off'),
@@ -316,6 +318,8 @@ ALL_ROUTES = [
     ('user:detail', 'SoftLayer.CLI.user.detail:cli'),
     ('user:permissions', 'SoftLayer.CLI.user.permissions:cli'),
     ('user:edit-permissions', 'SoftLayer.CLI.user.edit_permissions:cli'),
+    ('user:notifications', 'SoftLayer.CLI.user.notifications:cli'),
+    ('user:edit-notifications', 'SoftLayer.CLI.user.edit_notifications:cli'),
     ('user:edit-details', 'SoftLayer.CLI.user.edit_details:cli'),
     ('user:create', 'SoftLayer.CLI.user.create:cli'),
     ('user:delete', 'SoftLayer.CLI.user.delete:cli'),
