@@ -143,19 +143,6 @@ class VirtTests(testing.TestCase):
         result = self.run_command(['vs', 'list', '--tag=tag'])
 
         self.assert_no_fail(result)
-        self.assertEqual(json.loads(result.output),
-                         [{'datacenter': 'TEST00',
-                           'primary_ip': '172.16.240.2',
-                           'hostname': 'vs-test1',
-                           'action': None,
-                           'id': 100,
-                           'backend_ip': '10.45.19.37'},
-                          {'datacenter': 'TEST00',
-                           'primary_ip': '172.16.240.7',
-                           'hostname': 'vs-test2',
-                           'action': None,
-                           'id': 104,
-                           'backend_ip': '10.45.19.35'}])
 
     @mock.patch('SoftLayer.utils.lookup')
     def test_detail_vs_empty_billing(self, mock_lookup):
@@ -328,28 +315,13 @@ class VirtTests(testing.TestCase):
 
     def test_create_options(self):
         result = self.run_command(['vs', 'create-options'])
-
         self.assert_no_fail(result)
-        self.assertEqual({'cpus (dedicated host)': [4, 56],
-                          'cpus (dedicated)': [1],
-                          'cpus (standard)': [1, 2, 3, 4],
-                          'datacenter': ['ams01', 'dal05'],
-                          'flavors (balanced)': ['B1_1X2X25', 'B1_1X2X100'],
-                          'flavors (balanced local - hdd)': ['BL1_1X2X100'],
-                          'flavors (balanced local - ssd)': ['BL2_1X2X100'],
-                          'flavors (compute)': ['C1_1X2X25'],
-                          'flavors (memory)': ['M1_1X2X100'],
-                          'flavors (GPU)': ['AC1_1X2X100', 'ACL1_1X2X100'],
-                          'flavors (transient)': ['B1_1X2X25_TRANSIENT'],
-                          'local disk(0)': ['25', '100'],
-                          'memory': [1024, 2048, 3072, 4096],
-                          'memory (dedicated host)': [8192, 65536],
-                          'nic': ['10', '100', '1000'],
-                          'nic (dedicated host)': ['1000'],
-                          'os (CENTOS)': 'CENTOS_6_64',
-                          'os (DEBIAN)': 'DEBIAN_7_64',
-                          'os (UBUNTU)': 'UBUNTU_12_64'},
-                         json.loads(result.output))
+        self.assertIn('datacenter', result.output)
+        self.assertIn('flavor', result.output)
+        self.assertIn('memory', result.output)
+        self.assertIn('cpu', result.output)
+        self.assertIn('OS', result.output)
+        self.assertIn('network', result.output)
 
     @mock.patch('SoftLayer.CLI.formatting.confirm')
     def test_dns_sync_both(self, confirm_mock):
@@ -370,19 +342,19 @@ class VirtTests(testing.TestCase):
                                            'getResourceRecords')
         getResourceRecords.return_value = []
         createAargs = ({
-            'type': 'a',
-            'host': 'vs-test1',
-            'domainId': 12345,  # from SoftLayer_Account::getDomains
-            'data': '172.16.240.2',
-            'ttl': 7200
-        },)
+                           'type': 'a',
+                           'host': 'vs-test1',
+                           'domainId': 12345,  # from SoftLayer_Account::getDomains
+                           'data': '172.16.240.2',
+                           'ttl': 7200
+                       },)
         createPTRargs = ({
-            'type': 'ptr',
-            'host': '2',
-            'domainId': 123456,
-            'data': 'vs-test1.test.sftlyr.ws',
-            'ttl': 7200
-        },)
+                             'type': 'ptr',
+                             'host': '2',
+                             'domainId': 123456,
+                             'data': 'vs-test1.test.sftlyr.ws',
+                             'ttl': 7200
+                         },)
 
         result = self.run_command(['vs', 'dns-sync', '100'])
 
@@ -425,12 +397,12 @@ class VirtTests(testing.TestCase):
             }
         }
         createV6args = ({
-            'type': 'aaaa',
-            'host': 'vs-test1',
-            'domainId': 12345,
-            'data': '2607:f0d0:1b01:0023:0000:0000:0000:0004',
-            'ttl': 7200
-        },)
+                            'type': 'aaaa',
+                            'host': 'vs-test1',
+                            'domainId': 12345,
+                            'data': '2607:f0d0:1b01:0023:0000:0000:0000:0004',
+                            'ttl': 7200
+                        },)
         guest.return_value = test_guest
         result = self.run_command(['vs', 'dns-sync', '--aaaa-record', '100'])
         self.assert_no_fail(result)
@@ -570,6 +542,27 @@ class VirtTests(testing.TestCase):
         self.assertIn({'id': 1122}, order_container['prices'])
         self.assertEqual(order_container['virtualGuests'], [{'id': 100}])
 
+    @mock.patch('SoftLayer.CLI.formatting.confirm')
+    def test_upgrade_disk(self, confirm_mock):
+        confirm_mock.return_value = True
+        result = self.run_command(['vs', 'upgrade', '100', '--flavor=M1_64X512X100',
+                                   '--resize-disk=10', '1', '--resize-disk=10', '2'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Product_Order', 'placeOrder')
+        call = self.calls('SoftLayer_Product_Order', 'placeOrder')[0]
+        order_container = call.args[0]
+        self.assertEqual(799, order_container['presetId'])
+        self.assertIn({'id': 100}, order_container['virtualGuests'])
+        self.assertEqual(order_container['virtualGuests'], [{'id': 100}])
+
+    @mock.patch('SoftLayer.CLI.formatting.confirm')
+    def test_upgrade_disk_error(self, confirm_mock):
+        confirm_mock.return_value = True
+        result = self.run_command(['vs', 'upgrade', '100', '--flavor=M1_64X512X100',
+                                   '--resize-disk=1000', '1', '--resize-disk=10', '2'])
+        self.assertEqual(result.exit_code, 1)
+        self.assertIsInstance(result.exception, SoftLayerAPIError)
+
     @mock.patch('SoftLayer.CLI.formatting.confirm')
     def test_upgrade_with_flavor(self, confirm_mock):
         confirm_mock.return_value = True
@@ -582,6 +575,17 @@ class VirtTests(testing.TestCase):
         self.assertIn({'id': 100}, order_container['virtualGuests'])
         self.assertEqual(order_container['virtualGuests'], [{'id': 100}])
 
+    @mock.patch('SoftLayer.CLI.formatting.confirm')
+    def test_upgrade_with_add_disk(self, confirm_mock):
+        confirm_mock.return_value = True
+        result = self.run_command(['vs', 'upgrade', '100', '--add-disk=10', '--add-disk=10'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Product_Order', 'placeOrder')
+        call = self.calls('SoftLayer_Product_Order', 'placeOrder')[0]
+        order_container = call.args[0]
+        self.assertIn({'id': 100}, order_container['virtualGuests'])
+        self.assertEqual(order_container['virtualGuests'], [{'id': 100}])
+
     @mock.patch('SoftLayer.CLI.formatting.confirm')
     def test_upgrade_with_cpu_memory_and_flavor(self, confirm_mock):
         confirm_mock.return_value = True
@@ -815,3 +819,66 @@ class VirtTests(testing.TestCase):
         }
         self.assert_no_fail(result)
         self.assertEqual(json.loads(result.output), vir_billing)
+
+    def test_vs_migrate_list(self):
+        result = self.run_command(['vs', 'migrate'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Account', 'getVirtualGuests')
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrate')
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrateDedicatedHost')
+
+    def test_vs_migrate_list_empty(self):
+        mock = self.set_mock('SoftLayer_Account', 'getVirtualGuests')
+        mock.return_value = []
+        result = self.run_command(['vs', 'migrate'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Account', 'getVirtualGuests')
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrate')
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrateDedicatedHost')
+        self.assertIn("No guests require migration at this time", result.output)
+
+    def test_vs_migrate_guest(self):
+        result = self.run_command(['vs', 'migrate', '-g', '100'])
+        self.assert_no_fail(result)
+        self.assertIn('Started a migration on', result.output)
+        self.assert_not_called_with('SoftLayer_Account', 'getVirtualGuests')
+        self.assert_called_with('SoftLayer_Virtual_Guest', 'migrate', identifier=100)
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrateDedicatedHost')
+
+    def test_vs_migrate_all(self):
+        result = self.run_command(['vs', 'migrate', '-a'])
+        self.assert_no_fail(result)
+        self.assertIn('Started a migration on', result.output)
+        self.assert_called_with('SoftLayer_Virtual_Guest', 'migrate', identifier=100)
+        self.assert_called_with('SoftLayer_Virtual_Guest', 'migrate', identifier=104)
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrateDedicatedHost')
+
+    def test_vs_migrate_all_empty(self):
+        mock = self.set_mock('SoftLayer_Account', 'getVirtualGuests')
+        mock.return_value = []
+        result = self.run_command(['vs', 'migrate', '-a'])
+        self.assert_no_fail(result)
+        self.assertIn('No guests require migration at this time', result.output)
+
+    def test_vs_migrate_dedicated(self):
+        result = self.run_command(['vs', 'migrate', '-g', '100', '-h', '999'])
+        self.assert_no_fail(result)
+        self.assertIn('Started a migration on', result.output)
+        self.assert_not_called_with('SoftLayer_Account', 'getVirtualGuests')
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrate', identifier=100)
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrateDedicatedHost', args=(999), identifier=100)
+
+    def test_vs_migrate_exception(self):
+        ex = SoftLayerAPIError('SoftLayer_Exception', 'PROBLEM')
+        mock = self.set_mock('SoftLayer_Virtual_Guest', 'migrate')
+        mock.side_effect = ex
+        result = self.run_command(['vs', 'migrate', '-g', '100'])
+        self.assert_no_fail(result)
+        self.assertIn('Failed to migrate', result.output)
+        self.assert_not_called_with('SoftLayer_Account', 'getVirtualGuests')
+        self.assert_called_with('SoftLayer_Virtual_Guest', 'migrate', identifier=100)
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrateDedicatedHost', args=(999), identifier=100)
+
+    def test_list_vsi(self):
+        result = self.run_command(['vs', 'list', '--hardware'])
+        self.assert_no_fail(result)
