@@ -81,7 +81,7 @@ class NetworkTests(testing.TestCase):
         # Test a four public address IPv4 order
         result = self.network.add_subnet('public',
                                          quantity=4,
-                                         vlan_id=1234,
+                                         endpoint_id=1234,
                                          version=4,
                                          test_order=True)
 
@@ -89,7 +89,7 @@ class NetworkTests(testing.TestCase):
 
         result = self.network.add_subnet('public',
                                          quantity=4,
-                                         vlan_id=1234,
+                                         endpoint_id=1234,
                                          version=4,
                                          test_order=False)
 
@@ -104,7 +104,7 @@ class NetworkTests(testing.TestCase):
         # Test a public IPv6 order
         result = self.network.add_subnet('public',
                                          quantity=64,
-                                         vlan_id=45678,
+                                         endpoint_id=45678,
                                          version=6,
                                          test_order=True)
 
@@ -158,6 +158,26 @@ class NetworkTests(testing.TestCase):
         self.assert_called_with('SoftLayer_Billing_Item', 'cancelService',
                                 identifier=1056)
 
+    def test_set_tags_subnet(self):
+        subnet_id = 1234
+        tags = 'tags1,tag2'
+        result = self.network.set_tags_subnet(subnet_id, tags)
+
+        self.assertEqual(result, True)
+        self.assert_called_with('SoftLayer_Network_Subnet', 'setTags',
+                                identifier=subnet_id,
+                                args=(tags,))
+
+    def test_edit_note_subnet(self):
+        subnet_id = 1234
+        note = 'test note'
+        result = self.network.edit_note_subnet(subnet_id, note)
+
+        self.assertEqual(result, True)
+        self.assert_called_with('SoftLayer_Network_Subnet', 'editNote',
+                                identifier=subnet_id,
+                                args=(note,))
+
     def test_create_securitygroup(self):
         result = self.network.create_securitygroup(name='foo',
                                                    description='bar')
@@ -435,7 +455,7 @@ class NetworkTests(testing.TestCase):
 
     def test_resolve_subnet_ids(self):
         _id = self.network.resolve_subnet_ids('10.0.0.1/29')
-        self.assertEqual(_id, ['100'])
+        self.assertEqual(_id, ['100', '111'])
 
     def test_resolve_subnet_ids_no_results(self):
         mock = self.set_mock('SoftLayer_Account', 'getSubnets')
@@ -625,3 +645,12 @@ class NetworkTests(testing.TestCase):
         _filter = {'objectName': {'operation': 'CCI'}}
         self.assert_called_with('SoftLayer_Event_Log', 'getAllObjects', filter=_filter)
         self.assertEqual(100, log['accountId'])
+
+    def test_vlan_edit(self):
+        vlan_id = 100
+        name = "test"
+        note = "test note"
+        tags = "tag1,tag2"
+
+        self.network.edit(vlan_id, name, note, tags)
+        self.assert_called_with('SoftLayer_Network_Vlan', 'editObject')
